#include <vector>
#include <string>
#include <algorithm>
#include <ctime>

using namespace std;
#define undef 0x7F7F7F7F

void FillEdge(int x1, int y1, int x2, int y2);
void UnFillEdge(int x1, int y1, int x2, int y2);

vector<string> SplitByChar(string str, char separator)
{
	vector<string> res;
	int pos = 0;
	int len = 0;
	for (int i = 0; i < str.size(); i++)
	{
		if (str[i] == separator)
		{
			res.push_back(str.substr(pos, len));
			pos = i+1;
			len = 0;
		}
		else len++;
	}
	if (len != 0)
	{
		res.push_back(str.substr(pos, len));
	}
	return res;
}

int ParseInt(string str)
{
	int res = 0;
	for (int i = 0; i < str.size(); i++)
	{
		if (isdigit(str[i]))
			res = res*10 + (str[i] - '0');
	}
	return res;
}

vector<int> Split(string str)
{
	vector<string> args = SplitByChar(str, ' ');
	vector<int> res;
	for (int i = 0; i < args.size(); i++)
	{
		if (args[i].size() != 0)
			res.push_back(ParseInt(args[i]));
	}
	return res;
}

struct Point
{
	int x, y;

	Point(int x = 0, int y = 0)
	{
		this->x = x;
		this->y = y;
	}

	bool operator ==(Point p)
	{
		return x == p.x && y == p.y;
	}
};

const int maxstep = 1;

const int lines = 14;
const int columns = 20;
const int goalLength = 4;
const int goalStart = ((lines - goalLength) / 2);
const int goalFinish = goalStart + goalLength;
int ourColumn;
bool moves[lines+1][columns+1][lines+1][columns+1];
int x, y;

int MovesCount(int x, int y)
{
	int res = 0;
	for (int i = 0; i <= lines; i++)
		for (int j = 0; j <= columns; j++)
			if (moves[x][y][i][j])
				res++;
	return res;
}

void Fill(vector<Point> v)
{
	for (int i = 1; i < v.size(); i++)
	{
		moves[v[i].x][v[i].y][v[i - 1].x][v[i - 1].y] = true;
		moves[v[i - 1].x][v[i - 1].y][v[i].x][v[i].y] = true;
	}
	x = v[v.size() - 1].x;
	y = v[v.size() - 1].y;
}

void MakeEnemyMove(string str)
{
	vector<int> nums = Split(str);
	int moveLen = nums[0];
	vector<Point> points;
	for (int i = 0; i < moveLen; i++)
	{
		int x = nums[i * 2 + 1];
		int y = nums[i * 2 + 2];
		points.push_back(Point(x, y));
	}
	Fill(points);
}

bool mark[lines+1][columns+1];
Point parent[lines+1][columns+1];
int resx, resy;
int resDistToMyGoal;
int resDistToCenter;
int mdepth;
std::vector<int> Move;
std::vector<int> resMove;
int cost[50];
int mincost[50];

void InitDfs()
{
	memset(mark, 0, sizeof(mark));
	memset(parent, 0, sizeof(parent));
	resx = -1;
	resy = -1;
	resDistToMyGoal = columns + 2;
	resDistToCenter = lines + 2;
	mdepth = -1;
}

int d[8][2]  = {{-1, -1}, {-1, 0}, {-1, 1}, {0, -1}, {0, 1}, {1, -1}, {1, 0}, {1, 1}};

bool isGoal(int x, int y)
{
	return ((y == 0 || y == 20) && x > goalStart && x < goalFinish);
}

void Dfs(int curx, int cury, int lastx, int lasty, int step)
{
	mark[curx][cury] = true;
	parent[curx][cury] = Point(lastx, lasty);
	bool enemy = (step & 1);
	for (int dir = 0; dir < 8; dir++)
	{
		int dx = d[dir][0];
		int dy = d[dir][1];
		int x = curx + dx;
		int y = cury + dy;
		if (x < 0 || x > lines || y < 0 || y > columns) continue;
		if (!moves[curx][cury][x][y] && !mark[x][y])
		{
			FillEdge(curx, cury, x, y);
			bool _isGoal = isGoal(x, y);
			if (MovesCount(x, y) == 0 || _isGoal)
			{
				if (step == maxstep || _isGoal)
				{
					int distToMyGoal = abs(y - ourColumn);
					int distToCenter = abs(x - lines / 2);
					int randomValue = 0x1 & rand();
					int depth = 1024 - Move.size();
					int _cost = (((distToMyGoal<<3) + distToCenter)<<11) + (depth<<1) + randomValue;
					if (enemy)
						_cost = -_cost;
					if (_isGoal && y == ourColumn)
					{
						if (enemy)
							_cost = 0x7F000000;
						else
							_cost = -0x7F000000;
					} else if (_isGoal)
					{
						if (enemy)
							_cost = -0x7F000000;
						else
							_cost = 0x7F000000;
					}
					if (cost[step] == undef || _cost < cost[step])
					{
						cost[step] = _cost;
						if (!step)
						{
							resMove.assign(Move.begin(),Move.end());
							resMove.push_back(dir);
						}
					}
				}
				else
				{
					cost[step+1] = undef;
					Dfs(x, y, curx, cury, step+1);
					if (cost[step+1] == undef)
					{
						if (enemy)
							cost[step+1] = 0x7F000000;
						else
							cost[step+1] = -0x7F000000;
					}
					if (cost[step] == undef || -cost[step+1] < cost[step])
					{
						cost[step] = -cost[step+1];
						if (!step)
						{
							resMove.assign(Move.begin(),Move.end());
							resMove.push_back(dir);
						}
					}
				}
			}
			else
			{
				if (!step)
					Move.push_back(dir);
				Dfs(x, y, curx, cury, step);
				if (!step)
					Move.pop_back();
			}
			UnFillEdge(curx, cury, x, y);
		}
	}
	mark[curx][cury] = false;
}

vector<Point> CollectMove()
{
	vector<Point> res;
	res.push_back(Point(x,y));
	for (int i=0; i<resMove.size(); ++i)
	{
		Point p = Point(x + d[resMove[i]][0], y + d[resMove[i]][1]);
		x = p.x;
		y = p.y;
		res.push_back(p);
	}
	return res;
}

void MakeMove()
{
	InitDfs();
	cost[0] = undef;
	Dfs(x, y, -1, -1, 0);
	vector<Point> myMove = CollectMove();
	printf("%d", myMove.size());
	for (int i = 0; i < myMove.size(); i++)
		printf(" %d %d", myMove[i].x, myMove[i].y);
	printf("\n");
	Fill(myMove);
}

void FillEdge(int x1, int y1, int x2, int y2)
{
	if (x1 < 0 || x1 > lines) return;
	if (x2 < 0 || x2 > lines) return;
	if (y1 < 0 || y1 > columns) return;
	if (y2 < 0 || y2 > columns) return;
	moves[x1][y1][x2][y2] = true;
}

void UnFillEdge(int x1, int y1, int x2, int y2)
{
	if (x1 < 0 || x1 > lines) return;
	if (x2 < 0 || x2 > lines) return;
	if (y1 < 0 || y1 > columns) return;
	if (y2 < 0 || y2 > columns) return;
	moves[x1][y1][x2][y2] = false;
}

void InitField(bool enemyFirst)
{
	x = lines / 2;
	y = columns / 2;
	ourColumn = enemyFirst ? 0 : columns;

	for (int i = 0; i <= lines; i++)
	{
		if (i <= goalStart || i >= goalFinish)
		{
			FillEdge(i, 0, i - 1, 0);
			FillEdge(i, columns, i - 1, columns);
			FillEdge(i, 0, i + 1, 0);
			FillEdge(i, columns, i + 1, columns);
		}
	}
	for (int j = 0; j <= columns; j++)
	{
		FillEdge(0, j, 0, j - 1);
		FillEdge(0, j, 0, j + 1);
		FillEdge(lines, j, lines, j - 1);
		FillEdge(lines, j, lines, j + 1);
	}
}

char buf[1000];
char name[1000];
void Run() {
	bool firstMove = true;
	while (true) {
		gets(buf);
		string str = string(buf);
		if (str == "Quit")
			return;
		if (str == "Name") {
			printf("%s\n", name);
			fflush(stdout);
			continue;
		}
		if (str != "Start") {
			if (firstMove) {
				InitField(true);
				firstMove = false;
			}
			MakeEnemyMove(str);
		}
		if (firstMove) {
			InitField(false);
			firstMove = false;
		}
		MakeMove();
		fflush(stdout);
	}
}

int main(int argc, char **argv)
{
	srand(time(0));
	sprintf(name, "~~~~~~~~(m--)m", ""); //argv[0]);
	for (int i=0; name[i]; ++i)
	{
		if ((unsigned char)name[i]<32
		|| (unsigned char)name[i]>127)
			throw 42;
	}
	Run();
}