using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using System.Linq;

namespace CSharpBot
{
    public class Utils
    {
        public static List<int> Split(string str)
        {
            var args = str.Split(' ');
            var res = new List<int>();
            for (int i = 0; i < args.Length; i++)
            {
                if (!string.IsNullOrEmpty(args[i]))
                    res.Add(int.Parse(args[i]));
            }
            return res;
        }
    }

    public class Point
    {
        public int x, y;

        public Point(int x, int y)
        {
            this.x = x;
            this.y = y;
        }

        public bool Equals(Point other)
        {
            if (ReferenceEquals(null, other)) return false;
            if (ReferenceEquals(this, other)) return true;
            return other.x == x && other.y == y;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != typeof(Point)) return false;
            return Equals((Point)obj);
        }

        public override int GetHashCode()
        {
            unchecked
            {
                return (x * 397) ^ y;
            }
        }
    }

    public class Bot
    {
        public readonly int[,] d = new[,] { { -1, 0 }, { 1, 0 }, { 0, -1 }, { 0, 1 }, { -1, -1 }, { -1, 1 }, { 1, -1 }, { 1, 1 } };
        public const int columns = 20;
        public const int goalLength = 4;
        public const int lines = 14;
        public const int goalFinish = goalStart + goalLength;
        public const int goalStart = ((lines - goalLength) / 2);
        public bool[, , ,] moves;
        public bool[,] mark;
        public int ourColumn;
        public Point[,] parent;
        public int resDistToCenter;
        public int resDistToMyGoal;
        public int resx, resy;
        public int x, y;
        public HashSet<Tuple<Point, Point>> posMoves;

        public double oldBonus;
        
        public List<Tuple<int, int>> FinalPositions = new List<Tuple<int,int>>();


        public Bot()
        {
            moves = new bool[lines + 1, columns + 1, lines + 1, columns + 1];
            for (int i = 0; i <= lines; i++)
                for (int j = 0; j <= columns; j++)
                    for (int k = 0; k <= lines; k++)
                        for (int l = 0; l <= columns; l++)
                            moves[i, j, k, l] = false;
        }

        public Bot(bool[,,,] old_moves)
        {
            moves = old_moves;
        }

        public int MovesCount(int x, int y)
        {
            int res = 0;
            for (int i = 0; i <= lines; i++)
                for (int j = 0; j <= columns; j++)
                    if (moves[x, y, i, j])
                        res++;
            return res;
        }

        public void Fill(List<Point> v)
        {
            for (int i = 1; i < v.Count; i++)
            {
                moves[v[i].x, v[i].y, v[i - 1].x, v[i - 1].y] = true;
                moves[v[i - 1].x, v[i - 1].y, v[i].x, v[i].y] = true;
            }
            x = v[v.Count - 1].x;
            y = v[v.Count - 1].y;
        }

        public void MakeEnemyMove(String str)
        {
            var nums = Utils.Split(str);
            int moveLen = nums[0];
            var points = new List<Point>();
            for (int i = 0; i < moveLen; i++)
            {
                int x = nums[i * 2 + 1];
                int y = nums[i * 2 + 2];
                points.Add(new Point(x, y));
            }
            Fill(points);
        }
        
        public void InitDfs()
        {

            Log.WriteLine("init");
            mark = new bool[lines + 1, columns + 1];
            for (int i = 0; i <= lines; i++)
                for (int j = 0; j <= columns; j++)
                    mark[i, j] = false;

            parent = new Point[lines + 1, columns + 1];
            resx = -1;
            resy = -1;
            resDistToMyGoal = columns + 2;
            resDistToCenter = lines + 2;
            oldBonus = double.MaxValue;

            posMoves = new HashSet<Tuple<Point, Point>>();
        }

        public void Dfs(int curx, int cury, int lastx, int lasty)
        {
            mark[curx, cury] = true;
            parent[curx, cury] = new Point(lastx, lasty);
            for (int dir = 0; dir < 8; dir++)
            {
                int dx = d[dir, 0];
                int dy = d[dir, 1];
                int x = curx + dx;
                int y = cury + dy;
                if (x < 0 || x > lines || y < 0 || y > columns) continue;
                if (!moves[curx, cury, x, y] && !mark[x, y])
                {
                    if (MovesCount(x, y) == 0)
                    {
                        /*posMoves.Clear();

                        var list = CollectMove(x, y);
                        for(int i = 1; i < list.Count; i++)
                        {
                            posMoves.Add(new Tuple<Point, Point>(list[i-1], list[i]));
                        }

                        Dfs2(x,y,);*/

                        parent[x, y] = new Point(curx, cury);
                        mark[x, y] = true;



                        var bot = GetEnemyBot(x, y);
                        bot.InitDfs();
                        bot.Dfs2(bot.x, bot.y, -1, -1);

                        var mybot = bot.GetEnemyBot(bot.resx, bot.resy);
                        mybot.InitDfs();
                        mybot.Dfs2(bot.resx, bot.resy, -1, -1);

                        var bot1 = mybot.GetEnemyBot(mybot.resx, mybot.resy);
                        bot1.InitDfs();
                        bot1.Dfs2(mybot.resx, mybot.resy, -1, -1);
                        
                        var mybot1 = bot1.GetEnemyBot(bot1.resx, bot1.resy);
                        mybot1.InitDfs();
                        mybot1.Dfs2(bot1.resx, bot1.resy, -1, -1);

                        var bonus = BonusFunction(x, y)/2.0 - bot.oldBonus +
                            mybot.oldBonus*4 - bot1.oldBonus/1.5 + mybot1.oldBonus/4.0;
                        //Log.WriteLine(String.Format("{0} {1} {2}", x, y, bonus));

                        if (bonus < oldBonus)
                        {
                            resx = x;
                            resy = y;
                            oldBonus = bonus;
                        }
                    }
                    else
                    {
                        Dfs(x, y, curx, cury);
                    }
                }
            }
        }

        public void Dfs2(int curx, int cury, int lastx, int lasty)
        {
            mark[curx, cury] = true;
            parent[curx, cury] = new Point(lastx, lasty);
            for (int dir = 0; dir < 8; dir++)
            {
                int dx = d[dir, 0];
                int dy = d[dir, 1];
                int x = curx + dx;
                int y = cury + dy;
                if (x < 0 || x > lines || y < 0 || y > columns) continue;
                if (!moves[curx, cury, x, y] && !mark[x, y])
                {
                    if (MovesCount(x, y) == 0)
                    {
                        parent[x, y] = new Point(curx, cury);
                        mark[x, y] = true;
                        int distToMyGoal = Math.Abs(y - ourColumn);
                        //if (distToMyGoal == 0) distToMyGoal = columns + 1;
                        int distToCenter = Math.Abs(x - lines / 2);
                        
                        var bonus = BonusFunction(x, y);
                        if (bonus < oldBonus)
                        {
                            resx = x;
                            resy = y;
                            oldBonus = bonus;
                        }
                    }
                    else
                    {
                        Dfs2(x, y, curx, cury);
                    }
                }
            }
        }

        public double BonusFunction(int x, int y)
        {
            int distToMyGoal = Math.Abs(y - ourColumn);
            //if (distToMyGoal == 0) distToMyGoal = columns + 1;
            int distToCenter = Math.Abs(x - lines / 2);
            return distToMyGoal == 0 && distToCenter <= 2 
                ? -100500 
                : distToMyGoal == 20 && distToCenter <= 2 
                    ? 100500
                    : distToMyGoal + 0.001 * distToCenter;
        }

        public List<Point> CollectMove(int x, int y)
        {
            var res = new List<Point>();
            while (!(x == -1 && y == -1))
            {
                var p = new Point(x, y);
                res.Add(p);
                x = parent[p.x, p.y].x;
                y = parent[p.x, p.y].y;
            }
            var resArray = res.ToArray();
            Array.Reverse(resArray);
            res = new List<Point>(resArray);
            return res;
        }

        public void MakeMove()
        {
            InitDfs();
            Dfs(x, y, -1, -1);
            var myMove = CollectMove(resx, resy);
            Console.Write(myMove.Count);
            Log.Write(myMove.Count);
            for (int i = 0; i < myMove.Count; i++)
            {
                Console.Write(" " + myMove[i].x + " " + myMove[i].y);
                Log.Write(" " + myMove[i].x + " " + myMove[i].y);
            }
            Console.WriteLine();
            Log.WriteLine();
            Log.Flush();
            Fill(myMove);
        }

        public void FillEdge(int x1, int y1, int x2, int y2)
        {
            if (x1 < 0 || x1 > lines) return;
            if (x2 < 0 || x2 > lines) return;
            if (y1 < 0 || y1 > columns) return;
            if (y2 < 0 || y2 > columns) return;
            moves[x1, y1, x2, y2] = true;
        }

        public void InitField(bool enemyFirst)
        {
            x = lines / 2;
            y = columns / 2;
            ourColumn = enemyFirst ? columns : 0;

            for (int i = 0; i <= lines; i++)
            {
                if (i <= goalStart || i >= goalFinish)
                {
                    FillEdge(i, 0, i - 1, 0);
                    FillEdge(i, columns, i - 1, columns);
                    FillEdge(i, 0, i + 1, 0);
                    FillEdge(i, columns, i + 1, columns);
                }
            }
            for (int j = 0; j <= columns; j++)
            {
                FillEdge(0, j, 0, j - 1);
                FillEdge(0, j, 0, j + 1);
                FillEdge(lines, j, lines, j - 1);
                FillEdge(lines, j, lines, j + 1);
            }
        }

        public void Run()
        {
            bool firstMove = true;
            while (true)
            {
                String str = Console.ReadLine();
                if (str == "Quit")
                    return;
                if (str == "Name")
                {
                    Console.WriteLine("NoName5");
                    Console.Out.Flush();
                    continue;
                }
                if (str != "Start")
                {
                    if (firstMove)
                    {
                        InitField(false);
                        firstMove = false;
                    }
                    MakeEnemyMove(str);
                }
                if (firstMove)
                {
                    InitField(true);
                    firstMove = false;
                }
                MakeMove();
                Console.Out.Flush();
            }
        }

        public static StreamWriter Log;

        public static void Main(String[] args)
        {
            try
            {
                using (Log = new StreamWriter(new MemoryStream()))// StreamWriter(File.Create("log.txt")))
                {
                    new Bot().Run();

                }
            }
            catch(Exception e)
            {
                Log.WriteLine(e.Message);
                Log.WriteLine(e.StackTrace);
            }
        }

        public Bot GetEnemyBot(int x, int y)
        {
            var bot = new Bot((bool[, , ,])moves.Clone())
            {
                ourColumn = columns - ourColumn,
                x = x,
                y = y,
                mark = new bool[lines + 1, columns + 1]
            };

            bot.Fill(CollectMove(x,y));

            return bot;
        }
    }
}