#include <iostream>
#include <algorithm>
#include <vector>
#include <string>
#include <sstream>
#include <fstream>
#include <map>
#include <set>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <ctime>
#include <utility>
#include <cstring>


using namespace std;
typedef long long LL;

template<typename T>
inline T Abs(const T& x) { return x < 0 ? -x : x; }

vector<string> SplitByChar(string str, char separator)
{
    vector<string> res;
    int pos = 0;
    int len = 0;
    for (int i = 0; i < str.size(); i++)
    {
        if (str[i] == separator)
        {
            res.push_back(str.substr(pos, len));
            pos = i+1;
            len = 0;
        }
        else len++;
    }
    if (len != 0)
    {
        res.push_back(str.substr(pos, len));
    }
    return res;
}

int ParseInt(string str)
{
    int res = 0;
    for (int i = 0; i < str.size(); i++)
    {
        if (isdigit(str[i]))
            res = res*10 + (str[i] - '0');
    }
    return res;
}

vector<int> Split(string str)
{
    vector<string> args = SplitByChar(str, ' ');
    vector<int> res;
    for (int i = 0; i < args.size(); i++)
    {
        if (args[i].size() != 0)
            res.push_back(ParseInt(args[i]));
    }
    return res;
}


const int maxh = 14;
const int maxw = 20;

struct TPoint {
    int y, x;
    TPoint(int y_ = 0, int x_ = 0)
        : y(y_)
        , x(x_)
    { }
};

bool operator<(const TPoint& first, const TPoint& second) {
    return first.y < second.y || first.y == second.y && first.x < second.x;
}
bool operator==(const TPoint& first, const TPoint& second) {
    return first.y == second.y && first.x == second.x;
}


struct TField {
    TPoint ball;
    vector< TPoint > e[maxh+1][maxw+1];
    int state[maxh+1][maxw+1];
    int goalCol;
    TField() 
    { }
};

void AddEdge(TField& f, const TPoint first, const TPoint& second) {
    f.e[first.y][first.x].push_back(second);
}

void RemoveEdge(TField& f, const TPoint first, const TPoint& second) {
    vector<TPoint>::iterator it;
    it = find(f.e[first.y][first.x].begin(), f.e[first.y][first.x].end(), second);
    if (it != f.e[first.y][first.x].end())
        f.e[first.y][first.x].erase(it);
    it = find(f.e[second.y][second.x].begin(), f.e[second.y][second.x].end(), first);
    if (it != f.e[second.y][second.x].end())
        f.e[second.y][second.x].erase(it);
}

const int dy[8] = {+0, +0, +1, -1, +1, -1, +1, -1};
const int dx[8] = {-1, +1, +0, +0, +1, -1, -1, +1};

void InitField(TField& f, bool enemyFirst)
{
    f.ball = TPoint(maxh/2, maxw/2);
    f.goalCol = enemyFirst ? 0 : maxw;
    memset(f.state, 0, sizeof(f.state));
    f.state[f.ball.y][f.ball.x] = 1;
    for (int y = 0; y <= maxh; ++y)
        for (int x = 0; x <= maxw; ++x) {
            if (y == 0 || x == 0 || y == maxh || x == maxw)
                f.state[y][x] = 1;
            for (int i = 0; i < 8; ++i)
                if (y+dy[i] >= 0 && y+dy[i] <= maxh && x+dx[i] >= 0 && x+dx[i] <= maxw) {
                    if ((y != 0 && y != maxh || dy[i] != 0) && (x != 0 && x != maxw || dx[i] != 0))
                        AddEdge(f, TPoint(y, x), TPoint(y+dy[i], x+dx[i]));
                }
        }
    f.state[6][0] = 0;
    f.state[7][0] = 0;
    f.state[8][0] = 0;
    f.state[6][20] = 0;
    f.state[7][20] = 0;
    f.state[8][20] = 0;
}

void Fill(TField& f, const vector<TPoint>& points) {
    for (int i = 0; i+1 < points.size(); ++i)
        RemoveEdge(f, points[i], points[i+1]);
    f.state[points.back().y][points.back().x] = 1;
    f.ball = points.back();
}

inline int DistFromGoal(const TField& f, const TPoint& p) {
    return Abs(p.x - f.goalCol) + (p.y < 6 ? Abs(6-p.y) : 0) + (p.y > 8 ? p.y-8 : 0);
}
inline int DistFromMy(const TField& f, const TPoint& p) {
    return Abs(p.x - (20-f.goalCol)) + (p.y < 6 ? Abs(6-p.y) : 0) + (p.y > 8 ? p.y-8 : 0);
}


LL GOAL_WEIGHT = 1;
LL LENGTH_WEIGHT = -1;
LL MY_WEIGHT = 10;

LL OUR_WEIGHT = -1;
LL ENEMY_GOAL_WEIGHT = -1000;
LL ENEMY_GATE_WEIGHT = 10000;

inline LL GetQuality(TField& f, const vector<TPoint>& v) {
    if (DistFromGoal(f, v.back()) == 0)
        return -1000000000;
    return GOAL_WEIGHT * DistFromGoal(f, v.back()) + LENGTH_WEIGHT * v.size() + 
           (DistFromMy(f, v.back()) <= 4) * 40 - DistFromMy(f, v.back()) * GOAL_WEIGHT;    
}

inline LL GetQuality2(TField& f, const vector<TPoint>& v) {
    if (DistFromMy(f, v.back()) == 0)
        return -1000000000;
    if (v.size() == 1)
        return 1000000000;

    return ENEMY_GOAL_WEIGHT / (DistFromMy(f, v.back()) + 1) + ENEMY_GATE_WEIGHT / (DistFromGoal(f, v.back()) + 1);

//    return GOAL_WEIGHT * DistFromMy(f, v.back()) + LENGTH_WEIGHT * v.size() + 
//           (DistFromGoal(f, v.back()) <= 4) * 40 - DistFromGoal(f, v.back()) * GOAL_WEIGHT;    

    return DistFromMy(f, v.back());
}

const int BACK[8] = {};

inline int EdgeNumber(const TPoint& first, const TPoint& second) {
    return (first.y-second.y+1) * 3 + (first.x-second.x);
}

bool used[maxh+1][maxw+1][10];
vector<TPoint> cur;
vector<TPoint> cur2;
vector<TPoint> best;
vector<TPoint> best2;

void Dfs(TField& f) {
    TPoint last = cur.back();
    if (f.state[last.y][last.x] == 0) {
        if (best.empty() || GetQuality(f, cur) < GetQuality(f, best))
            best = cur;
    } else {
        vector<TPoint>& edges = f.e[last.y][last.x];
        for (int i = 0; i < edges.size(); ++i)
            if (!used[last.y][last.x][EdgeNumber(last, edges[i])]) {
                used[last.y][last.x][EdgeNumber(last, edges[i])] = 1;
                used[edges[i].y][edges[i].x][EdgeNumber(edges[i], last)] = 1;
                cur.push_back(edges[i]);
                Dfs(f);
                cur.pop_back();
                used[last.y][last.x][EdgeNumber(last, edges[i])] = 0;
                used[edges[i].y][edges[i].x][EdgeNumber(edges[i], last)] = 0;
            }
    }
}

void Dfs2(TField& f) {
    TPoint last = cur2.back();
    if (f.state[last.y][last.x] == 0) {
        if (best2.empty() || GetQuality2(f, cur2) < GetQuality2(f, best2))
            best2 = cur2;
    } else {
        vector<TPoint>& edges = f.e[last.y][last.x];
        for (int i = 0; i < edges.size(); ++i)
            if (!used[last.y][last.x][EdgeNumber(last, edges[i])]) {
                used[last.y][last.x][EdgeNumber(last, edges[i])] = 1;
                used[edges[i].y][edges[i].x][EdgeNumber(edges[i], last)] = 1;
                cur2.push_back(edges[i]);
                Dfs2(f);
                cur2.pop_back();
                used[last.y][last.x][EdgeNumber(last, edges[i])] = 0;
                used[edges[i].y][edges[i].x][EdgeNumber(edges[i], last)] = 0;
            }
    }
}

const LL inf = 2000000000;
LL B = -inf;

void Dfs1(TField& f) {
    TPoint last = cur.back();
    if (f.state[last.y][last.x] == 0) {
        if (DistFromMy(f, cur.back()) == 0) {
            if (B == -inf) {
                B = -inf + 1;
                best = cur;
            } else
                return;
        }
        if (DistFromGoal(f, cur.back()) == 0) {
            best = cur;
            B = inf;
            return;
        }
        f.state[last.y][last.x] = 1;
        TPoint oldBall = f.ball;
        f.ball = last;
        best2.clear();
        cur2.clear();
        cur2.push_back(f.ball);
        Dfs2(f);
        f.ball = oldBall;
        f.state[last.y][last.x] = 0;
        if (B == -inf || B < GetQuality2(f, best2) - GetQuality(f, cur)) {
            best = cur;
            B = GetQuality2(f, best2) - GetQuality(f, cur);
        }
    } else {
        vector<TPoint>& edges = f.e[last.y][last.x];
        for (int i = 0; i < edges.size(); ++i)
            if (!used[last.y][last.x][EdgeNumber(last, edges[i])]) {
                used[last.y][last.x][EdgeNumber(last, edges[i])] = 1;
                used[edges[i].y][edges[i].x][EdgeNumber(edges[i], last)] = 1;
                cur.push_back(edges[i]);
                Dfs1(f);
                cur.pop_back();
                used[last.y][last.x][EdgeNumber(last, edges[i])] = 0;
                used[edges[i].y][edges[i].x][EdgeNumber(edges[i], last)] = 0;
            }
    }
}




void MakeMove(TField& f)
{
    B = -inf;
    memset(used, 0, sizeof(used));
    best.clear();
    cur.clear();
    cur.push_back(f.ball);
    Dfs1(f);
    vector<TPoint> myMove = best;
    printf("%d", myMove.size());
    for (int i = 0; i < myMove.size(); i++)
        printf(" %d %d", myMove[i].y, myMove[i].x);
    printf("\n");
    Fill(f, myMove);
}

void MakeEnemyMove(TField& f, string str)
{
    vector<int> nums = Split(str);
    int moveLen = nums[0];
    vector<TPoint> points;
    for (int i = 0; i < moveLen; i++)
    {
        int x = nums[i * 2 + 1];
        int y = nums[i * 2 + 2];
        points.push_back(TPoint(x, y));
    }
    Fill(f, points);
}

string name;
char buf[1000];
void Run() {
    TField f;
    bool firstMove = true;  
    while (true) {
        gets(buf);
        string str = string(buf);
        if (str == "Quit")
            return;
        if (str == "Name") {
            //printf("Moscow SU Room 315 2\n");
            printf("%s\n", name.c_str());
            fflush(stdout);
            continue;
        }
        if (str != "Start") {
            if (firstMove) {
                InitField(f, true);
                firstMove = false;
            }
            MakeEnemyMove(f, str);
        }
        if (firstMove) {
            InitField(f, false);
            firstMove = false;
        }
        MakeMove(f);
        fflush(stdout);
    }
}


int main() {
    name = "Moscow SU Room 315";
//    ifstream in("input1.txt");
//    in >> name >> GOAL_WEIGHT >> LENGTH_WEIGHT;    
//    in >> name >> OUR_WEIGHT >> ENEMY_GOAL_WEIGHT >> ENEMY_GATE_WEIGHT;

    Run();    

    return 0;
}
