#include <vector>
#include <string>
#include <algorithm>

using namespace std;

vector<string> SplitByChar(string str, char separator)
{
	vector<string> res;
	int pos = 0;
	int len = 0;
	for (int i = 0; i < str.size(); i++)
	{
		if (str[i] == separator)
		{
			res.push_back(str.substr(pos, len));
			pos = i+1;
			len = 0;
		}
		else len++;
	}
	if (len != 0)
	{
		res.push_back(str.substr(pos, len));
	}
	return res;
}

int ParseInt(string str)
{
	int res = 0;
	for (int i = 0; i < str.size(); i++)
	{
		if (isdigit(str[i]))
			res = res*10 + (str[i] - '0');
	}
	return res;
}

vector<int> Split(string str)
{
	vector<string> args = SplitByChar(str, ' ');
	vector<int> res;
	for (int i = 0; i < args.size(); i++)
	{
		if (args[i].size() != 0)
			res.push_back(ParseInt(args[i]));
	}
	return res;
}

struct Point
{
	int x, y;

	Point(int x = 0, int y = 0)
	{
		this->x = x;
		this->y = y;
	}

	bool operator ==(Point p)
	{
		return x == p.x && y == p.y;
	}
};

const int lines = 14;
const int columns = 20;
const int goalLength = 4;
const int goalStart = ((lines - goalLength) / 2);
const int goalFinish = goalStart + goalLength;
int ourColumn;
bool moves[lines+1][columns+1][lines+1][columns+1];
int x, y;

int MovesCount(int x, int y)
{
	int res = 0;
	for (int i = 0; i <= lines; i++)
		for (int j = 0; j <= columns; j++)
			if (moves[x][y][i][j])
				res++;
	return res;
}

void Fill(vector<Point> v)
{
	for (int i = 1; i < v.size(); i++)
	{
		moves[v[i].x][v[i].y][v[i - 1].x][v[i - 1].y] = true;
		moves[v[i - 1].x][v[i - 1].y][v[i].x][v[i].y] = true;
	}
	x = v[v.size() - 1].x;
	y = v[v.size() - 1].y;
}

void MakeEnemyMove(string str)
{
	vector<int> nums = Split(str);
	int moveLen = nums[0];
	vector<Point> points;
	for (int i = 0; i < moveLen; i++)
	{
		int x = nums[i * 2 + 1];
		int y = nums[i * 2 + 2];
		points.push_back(Point(x, y));
	}
	Fill(points);
}

bool mark[lines+1][columns+1];
Point parent[lines+1][columns+1];
int resx, resy;
int resDistToMyGoal;
int resDistToCenter;

void InitDfs()
{
	memset(mark, 0, sizeof(mark));
	memset(parent, 0, sizeof(parent));
	resx = -1;
	resy = -1;
	resDistToMyGoal = -1;
}

int d[8][2]  = {{-1, -1}, {-1, 0}, {-1, 1}, {0, -1}, {0, 1}, {1, -1}, {1, 0}, {1, 1}};


void Dfs(int curx, int cury, int lastx, int lasty)
{
	vector<pair<int, int> > v;
	for(int i = 0; i < 8; i++)
		v.push_back(make_pair(d[i][0], d[i][1]));
	random_shuffle(v.begin(), v.end());
	mark[curx][cury] = true;
	parent[curx][cury] = Point(lastx, lasty);
	for (int dir = 0; dir < 8; dir++)
	{
		int dx = v[dir].first;
		int dy = v[dir].second;
		int x = curx + dx;
		int y = cury + dy;
		if (x < 0 || x > lines || y < 0 || y > columns) continue;
		if (!moves[curx][cury][x][y] && !mark[x][y])
		{
			if (MovesCount(x, y) == 0)
			{
				parent[x][y] = Point(curx, cury);
				mark[x][y] = true;
				int distToMyGoal = abs(y - ourColumn);
				int distToCenter = 0;
				for(int q = 0; q < 8; q++)
				{
					int x2 = x + d[q][0];
					int y2 = y + d[q][1];
					
					if (x2 < 0 || x2 > lines || y2 < 0 || y2 > columns) continue;		               
					distToCenter += MovesCount(x2, y2);				
				}
				if (distToMyGoal > resDistToMyGoal || (distToMyGoal == resDistToMyGoal && distToCenter < resDistToCenter))
				{
					resx = x;
					resy = y;
					resDistToMyGoal = distToMyGoal;
					resDistToCenter = distToCenter;
				}
			}
			else
			{
				Dfs(x, y, curx, cury);
			}
		}
	}
}

vector<Point> CollectMove(int x, int y)
{
	vector<Point> res;
	while (!(x == -1 && y == -1))
	{
		Point p = Point(x, y);
		res.push_back(p);
		x = parent[p.x][p.y].x;
		y = parent[p.x][p.y].y;
	}
	reverse(res.begin(), res.end());
	return res;
}

void MakeMove()
{
	InitDfs();
	Dfs(x, y, -1, -1);
	vector<Point> myMove = CollectMove(resx, resy);
	printf("%d", myMove.size());
	for (int i = 0; i < myMove.size(); i++)
		printf(" %d %d", myMove[i].x, myMove[i].y);
	printf("\n");
	Fill(myMove);
}

void FillEdge(int x1, int y1, int x2, int y2)
{
	if (x1 < 0 || x1 > lines) return;
	if (x2 < 0 || x2 > lines) return;
	if (y1 < 0 || y1 > columns) return;
	if (y2 < 0 || y2 > columns) return;
	moves[x1][y1][x2][y2] = true;
}

void InitField(bool enemyFirst)
{
	x = lines / 2;
	y = columns / 2;
	ourColumn = enemyFirst ? columns : 0;

	for (int i = 0; i <= lines; i++)
	{
		if (i <= goalStart || i >= goalFinish)
		{
			FillEdge(i, 0, i - 1, 0);
			FillEdge(i, columns, i - 1, columns);
			FillEdge(i, 0, i + 1, 0);
			FillEdge(i, columns, i + 1, columns);
		}
	}
	for (int j = 0; j <= columns; j++)
	{
		FillEdge(0, j, 0, j - 1);
		FillEdge(0, j, 0, j + 1);
		FillEdge(lines, j, lines, j - 1);
		FillEdge(lines, j, lines, j + 1);
	}
}

char buf[1000];
void Run() {
	bool firstMove = true;	
	while (true) {
		gets(buf);
		string str = string(buf);
		if (str == "Quit")
			return;
		if (str == "Name") {
			printf("NelepyjICat\n");
			fflush(stdout);
			continue;
		}
		if (str != "Start") {
			if (firstMove) {
				InitField(true);
				firstMove = false;
			}
			MakeEnemyMove(str);
		}
		if (firstMove) {
			InitField(false);
			firstMove = false;
		}
		MakeMove();
		fflush(stdout);
	}
}

int main()
{
	Run();
}