#include <vector>
#include <fstream>
#include <queue>
#include <iostream>
#include <string>
#include <algorithm>
using namespace std;

const int lines = 14;
const int columns = 20;
const int n = lines + 1;
const int m = columns + 1;
const int goalLength = 4;
const int goalStart = ((lines - goalLength) / 2);
const int goalFinish = goalStart + goalLength;

char buf[1000];

int ourColumn;
int enemyColumn;
bool to[n][m][n][m];
bool used[n][m];
bool used2[n][m];
unsigned int bfsmas[n][m];
int yc, xc;

void FillEdge(int x1, int y1, int x2, int y2)
{
	if (x1 < 0 || x1 > lines) return;
	if (x2 < 0 || x2 > lines) return;
	if (y1 < 0 || y1 > columns) return;
	if (y2 < 0 || y2 > columns) return;
	to[x1][y1][x2][y2] = true;
}
vector<string> SplitByChar(string str, char separator)
{
	vector<string> res;
	int pos = 0;
	int len = 0;
	for (int i = 0; i < str.size(); i++)
	{
		if (str[i] == separator)
		{
			res.push_back(str.substr(pos, len));
			pos = i+1;
			len = 0;
		}
		else len++;
	}
	if (len != 0)
	{
		res.push_back(str.substr(pos, len));
	}
	return res;
}
int ParseInt(string str)
{
	int res = 0;
	for (int i = 0; i < str.size(); i++)
	{
		if (isdigit(str[i]))
			res = res*10 + (str[i] - '0');
	}
	return res;
}
vector<int> Split(string str)
{
	vector<string> args = SplitByChar(str, ' ');
	vector<int> res;
	for (int i = 0; i < args.size(); i++)
	{
		if (args[i].size() != 0)
			res.push_back(ParseInt(args[i]));
	}
	return res;
}
struct Point
{
	int x, y;

	Point(int x = 0, int y = 0)
	{
		this->x = x;
		this->y = y;
	}

	bool operator ==(Point p)
	{
		return x == p.x && y == p.y;
	}
};
void InitField(bool enemyFirst)
{
	yc = lines / 2;
	xc = columns / 2;
	used[yc][xc] = true;
	ourColumn = enemyFirst ? columns : 0;
	enemyColumn  = columns - ourColumn;

	for (int i = 0; i <= lines; i++)
	{
		if (i <= goalStart || i >= goalFinish)
		{
			used[i][0] = used[i][columns] = true;
			FillEdge(i, 0, i - 1, 0);
			FillEdge(i, columns, i - 1, columns);
			FillEdge(i, 0, i + 1, 0);
			FillEdge(i, columns, i + 1, columns);
		}
	}
	for (int j = 0; j <= columns; j++)
	{
		used[0][j] = used[lines][j] = true;
		FillEdge(0, j, 0, j - 1);
		FillEdge(0, j, 0, j + 1);
		FillEdge(lines, j, lines, j - 1);
		FillEdge(lines, j, lines, j + 1);
	}

}
void Fill(const vector<Point> &v)
{
	for (int i = 1; i < v.size(); i++)
	{
		to[v[i].x][v[i].y][v[i - 1].x][v[i - 1].y] = true;
		to[v[i - 1].x][v[i - 1].y][v[i].x][v[i].y] = true;
	}
	yc = v[v.size() - 1].x;
	xc = v[v.size() - 1].y;
	used[yc][xc]=true;
}
void MakeEnemyMove(const string &str)
{
	vector<int> nums = Split(str);
	int moveLen = nums[0];
	vector<Point> points;
	for (int i = 0; i < moveLen; i++)
	{
		int x = nums[i * 2 + 1];
		int y = nums[i * 2 + 2];
		points.push_back(Point(x, y));
	}
	Fill(points);
}

//----------------------------------------------------------------------------
int dy[]={-1,-1,-1,0,1,1,1,0};
int dx[]={-1,0,1,1,1,0,-1,-1};
pair<int,int> pr[n][m];
//vector<pair<int,int> > st;
pair<int,int> st[10000];
int stp=0;
//vector<pair<int,int> > best;
pair<int,int> best[10000];
int bestp=0;
double mindist;

void dfs(int y,int x)
{
	st[stp++]=(make_pair(y,x));
	if (!used[y][x])
	{
		double dist=bfsmas[y][x];

		/*
		if (y>goalStart && y<goalFinish)
			dist=abs(x-enemyColumn);
		else if (y<=goalStart)
			dist = hypot(y-6,x-enemyColumn);
		else
			dist = hypot(y-8,x-enemyColumn);
		//*/
		if (dist<mindist)
		{
			mindist=dist;
			for (int i=0;i<stp;i++)
				best[i]=st[i];
			bestp=stp;
		}
		stp--;
		return;
	}
	for (int l=0;l<8;l++)
	{
		int yy=y+dy[l];
		int xx=x+dx[l];
		if (yy<0 || yy>lines || xx<0||xx>columns)
			continue;

		if (!to[y][x][yy][xx])
		{
			to[y][x][yy][xx]=true;
			to[yy][xx][y][x]=true;
			dfs(yy,xx);
			to[y][x][yy][xx]=false;
			to[yy][xx][y][x]=false;
		}
	}
	stp--;
}
inline void f(unsigned int &x,unsigned int c,bool &was)
{
	if (x>c)
	{
		x=c;
		was=true;
	}
}
void MakeMove()
{
	//FILE* out = fopen("output.txt","w");
	queue<pair<int,int> > q;
	const int inf=1e6;
	for (int i=0;i<n;i++)
		for (int j=0;j<m;j++)
			bfsmas[i][j]=inf;
	for (int i=goalStart+1;i<goalFinish;i++)
	{
		q.push(make_pair(i,enemyColumn));
		bfsmas[i][enemyColumn]=0;
	}
	while (!q.empty())
	{
		int y=q.front().first;
		int x=q.front().second;
		q.pop();
		for (int l=0;l<8;l++)
		{
			int yy=y+dy[l];
			int xx=x+dx[l];
			if (yy<0 || yy>lines || xx<0 || xx>columns || used[yy][xx])
				continue;

			if (bfsmas[yy][xx]==inf)
			{
				bfsmas[yy][xx]=bfsmas[y][x]+1;
				q.push(make_pair(yy,xx));
			}

		}
	}

	//for (int i=0;i<n;i++){
	//	for (int j=0;j<m;j++)
	//		fprintf(out,"%10u ",bfsmas[i][j]);
	//	fprintf(out,"\n");
	//}

	bool was=true;
	while (was)
	{
		was=false;
		for (int y=0;y<n;y++)
			for (int x=0;x<m;x++)
			{
				for (int l=0;l<8;l++)
				{
					int yy=y+dy[l];
					int xx=x+dx[l];
					if (yy<0 || yy>lines || xx<0 || xx>columns)
						continue;

					if (used[y][x] && used[yy][xx])
					{
						if (!to[y][x][yy][xx])
							f(bfsmas[y][x],bfsmas[yy][xx],was);
					}
					else
						f(bfsmas[y][x],bfsmas[yy][xx]+1,was);
				}
			}
	}

	
	/*for (int i=0;i<n;i++){
	for (int j=0;j<m;j++)
		fprintf(out,"%10u ",bfsmas[i][j]);
		fprintf(out,"\n");
	}*/
	//fclose(out);
	for (int i=0;i<n;i++)
		for (int j=0;j<m;j++)
			pr[i][j]=make_pair(-1,-1);
	q.push(make_pair(yc,xc));
	memset(used2,0,sizeof(used2));
	used2[yc][xc]=1;
	
	while (!q.empty())
	{
		int y=q.front().first;
		int x=q.front().second;
		q.pop();

		for (int l=0;l<8;l++)
		{
			int yy=y+dy[l];
			int xx=x+dx[l];
			if (yy<0 || yy>lines || xx<0 || xx>columns || to[y][x][yy][xx] || used2[yy][xx])
				continue;

			if (!used[yy][xx])
			{
				used2[yy][xx]=true;
				pr[yy][xx]=make_pair(y,x);
			}

			if (!used2[yy][xx])
			{
				used2[yy][xx]=true;
				pr[yy][xx]=make_pair(y,x);
				q.push(make_pair(yy,xx));
			}

		}
	}

	int mindist=1e6;
	pair<int,int> good;
	int tmpdist=1e6;
	for (int i=0;i<n;i++)
		for (int j=0;j<m;j++)
			if (used2[i][j] && !used[i][j] && (bfsmas[i][j]<mindist || bfsmas[i][j]==mindist && abs(j-enemyColumn)<tmpdist))
			{
				tmpdist=abs(j-enemyColumn);
				mindist=bfsmas[i][j];
				good=make_pair(i,j);
			}

	vector<pair<int,int> > path;
	for (;good.first!=-1;)
	{
		pair<int,int> tmp=pr[good.first][good.second];
		path.push_back(good);
		good=tmp;
	}

	reverse(path.begin(),path.end());

	/*mindist=1e9;
	dfs(yc,xc);*/

	printf("%d",path.size());
	for (int i=0;i<path.size();i++)
	{
		printf(" %d %d",path[i].first,path[i].second);
		if (i){
			int y1=path[i].first;
			int x1=path[i].second;
			int y2=path[i-1].first;
			int x2=path[i-1].second;
			to[y1][x1][y2][x2]=to[y2][x2][y1][x1]=true;
		}
	}
	used[path.back().first][path.back().second] = true;
	printf("\n");
	return;

	printf("%d",bestp);
	for (int i=0;i<bestp;i++){
		printf(" %d %d",best[i].first,best[i].second);
		if (i){
			int y1=best[i].first;
			int x1=best[i].second;
			int y2=best[i-1].first;
			int x2=best[i-1].second;
			to[y1][x1][y2][x2]=to[y2][x2][y1][x1]=true;
		}
	}
	used[best[bestp-1].first][best[bestp-1].second] = true;
	printf("\n");
}

//----------------------------------------------------------------------------
void Run() {
	bool firstMove = true;	
	while (true) {
		gets(buf);
		string str = string(buf);
		if (str == "Quit")
			return;
		if (str == "Name") {
			printf("Pure\n");
			fflush(stdout);
			continue;
		}
		if (str != "Start") {
			if (firstMove) {
				InitField(true);
				firstMove = false;
			}
			MakeEnemyMove(str);
		}
		if (firstMove) {
			InitField(false);
			firstMove = false;
		}
		MakeMove();
		fflush(stdout);
	}
}

int main()
{
	Run();
}